/*
 *  arch/arm/mach-vt8500/cpuidle.c
 *
 *  Copyright (C) 2011 Tony Prisk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/cpuidle.h>
#include <linux/io.h>
#include <asm/proc-fns.h>

#define WMT_MAX_STATES	0x01

static DEFINE_PER_CPU(struct cpuidle_device, wmt_cpuidle_device);

static int wmt_enter_idle(struct cpuidle_device *dev,
				struct cpuidle_state *state)
{
	struct timeval before, after;
	int idle_time;

	local_irq_disable();
	do_gettimeofday(&before);

	if (state == &dev->states[0])
		cpu_do_idle();

	do_gettimeofday(&after);
	local_irq_enable();
	idle_time = (after.tv_sec - before.tv_sec) * USEC_PER_SEC + 
			(after.tv_usec - before.tv_usec);
	return idle_time;
}

static struct cpuidle_driver wmt_cpuidle_driver = {
	.name		= "wmt_idle",
	.owner		= THIS_MODULE,
};

static int wmt_cpuidle_init(void)
{
	struct cpuidle_device *device;

	cpuidle_register_driver(&wmt_cpuidle_driver);

	device = &per_cpu(wmt_cpuidle_device, smp_processor_id());

	device->state_count = WMT_MAX_STATES;

	/* wait for interrupt state */
	device->states[0].enter = wmt_enter_idle;
	device->states[0].exit_latency = 1;
	device->states[0].target_residency = 10000;
	device->states[0].flags = CPUIDLE_FLAG_TIME_VALID;
	strcpy(device->states[0].name, "WFI");
	strcpy(device->states[0].desc, "Wait for interrupt");

	if (cpuidle_register_device(device)) {
		printk(KERN_ERR "[cpuidle] Failed registering\n");
		return -EIO;
	}

	return 0;
}

device_initcall(wmt_cpuidle_init);
