/*
 *  /arch/arm/mach-vt8500/include/mach/i2c.h
 *
 *  Copyright (C) 2008 WonderMedia Technologies, Inc.
 *  Copyright (C) 2011 Tony Prisk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _WMT_I2C_H_
#define _WMT_I2C_H_

#define I2C0_BASE_ADDR		0xD8280000
#define I2C1_BASE_ADDR		0xD8320000

#define WMT_I2C_API_I2C_ADDR	0x59

/* I2C Register offsets */
#define I2C_CR                     0x0000
#define I2C_TCR                    0x0002
#define I2C_CSR                    0x0004
#define I2C_ISR                    0x0006
#define I2C_IMR                    0x0008
#define I2C_CDR                    0x000A
#define I2C_TR                     0x000C
#define I2C_DIV                    0x000E

#define I2C_SCR                    0x0010
#define I2C_SSR                    0x0012
#define I2C_SISR                   0x0014
#define I2C_SIMR                   0x0016
#define I2C_SDR                    0x0018
#define I2C_STR                    0x001A

/* Register Bit Fields */

#define I2C_CR_CPU_RDY                  0x0008
#define I2C_CR_TX_END                   0x0004
#define I2C_CR_TX_NEXT_NO_ACK           0x0002
#define I2C_CR_TX_NEXT_ACK              0x0000
#define I2C_CR_ENABLE                   0x0001
#define I2C_SLAV_MODE_SEL		0x8000

#define I2C_TCR_HS_MODE			0x2000
#define I2C_TCR_STANDARD_MODE           0x0000
#define I2C_TCR_FAST_MODE               0x8000
#define I2C_TCR_MASTER_WRITE            0x0000
#define I2C_TCR_MASTER_READ             0x4000
#define I2C_TCR_SLAVE_ADDR_MASK         0x007F

#define I2C_CSR_READY			0x0002
#define I2C_CSR_BUSY			0x0000
#define I2C_CSR_STATUS_MASK		0x0002
#define I2C_CSR_RCV_ACK                 0x0000
#define I2C_CSR_RCV_NOT_ACK             0x0001
#define I2C_CSR_RCV_ACK_MASK            0x0001

#define I2C_ISR_SCL_TIMEOUT		0x0004
#define I2C_ISR_SCL_TIMEOUT_WRITE_CLEAR	0x0004
#define I2C_ISR_BYTE_END		0x0002
#define I2C_ISR_BYTE_END_WRITE_CLEAR	0x0002
#define I2C_ISR_NACK_ADDR		0x0001
#define I2C_ISR_NACK_ADDR_WRITE_CLEAR	0x0001
#define I2C_ISR_ALL_WRITE_CLEAR		0x0007

#define I2C_IMR_SCL_TIMEOUT_MASK	0x0004
#define I2C_IMR_BYTE_END_MASK           0x0002
#define I2C_IMR_NACK_ADDR_MASK          0x0001
#define I2C_IMR_ALL_ENABLE              0x0007

#define I2C_CDR_DATA_READ_MASK          0xFF00
#define I2C_CDR_DATA_WRITE_MASK         0x00FF

#define I2C_TR_SCL_TIMEOUT_MASK		0xFF00
#define I2C_TR_FSTP_MASK                0x00FF

#define I2C_TR_STD_VALUE                0x8041
#define I2C_TR_FAST_VALUE               0x8011

#define APB_96M_I2C_DIV                 7
#define APB_166M_I2C_DIV                12


/* I2C slave registers setting */
#define HS_MASTER_CODE			0x0800

#define I2C_SLAVE_ADDR 			0x59
#define I2C_SLAVE_MASK 			0x007F
#define I2C_SLAVE_NACK 			(1 << 12)
#define I2C_SLAVE_HS_MODE		(1 << 14)
#define I2C_SLAVE_EN			(1 << 15)

#define I2C_SISR_SCL_TIMEOUT			0x0004
#define I2C_SISR_SCL_TIMEOUT_WRITE_CLEAR	0x0004
#define I2C_SISR_BYTE_END			0x0002
#define I2C_SISR_BYTE_END_WRITE_CLEAR		0x0002
#define I2C_SISR_DAT_REQ			0x0001
#define I2C_SISR_DAT_REQ_WRITE_CLEAR		0x0001
#define I2C_SISR_ALL_WRITE_CLEAR		0x0007

#define I2C_SIMR_SCL_TIMEOUT_MASK	0x0004
#define I2C_SIMR_BYTE_END_MASK		0x0002
#define I2C_SIMR_NACK_ADDR_MASK		0x0001

#define I2C_SIMR_ALL_ENABLE		0x0007

#define I2C_SRCV_NACK			0x01
#define I2C_SREAD			0x02
#define I2C_SACT			0x04

#define I2C_SLAVE_WRITE_DATA_SHIFT	0
#define I2C_SLAVE_READ_DATA_SHIFT	8
#define I2C_SLAVE_READ_DATA_MASK	0xFF00
#define I2C_SLAVE_WRITE_DATA_MASK	0x00FF

enum i2c_mode_e {
	I2C_STANDARD_MODE = 0 ,
	I2C_FAST_MODE     = 1,
	I2C_HS_MODE	= 2,
};

struct i2c_regs_s {
	volatile unsigned short cr_reg;
	volatile unsigned short tcr_reg;
	volatile unsigned short csr_reg;
	volatile unsigned short isr_reg;
	volatile unsigned short imr_reg;
	volatile unsigned short cdr_reg;
	volatile unsigned short tr_reg;
	volatile unsigned short div_reg;
	volatile unsigned short scr_reg;
	volatile unsigned short cssr_reg;
	volatile unsigned short sisr_reg;
	volatile unsigned short simr_reg;
	volatile unsigned short csdr_reg;
	volatile unsigned short str_reg;
};

#define SUSPEND_NOTIFY		0
#define SUSPEND_SAVE_STATE	1
#define SUSPEND_DISABLE		2
#define SUSPEND_POWER_DOWN	3
#define RESUME_POWER_ON		0
#define RESUME_RESTORE_STATE	1
#define RESUME_ENABLE		2

#define I2C_ALGO_WMT		0x00900000

#define I2C_ADAPTER_RETRIES	3
#define I2C_ALGO_UDELAY		10
#define I2C_ALGO_TIMEOUT	500

#define MAX_MESSAGES            65536

#define I2C_SET_STANDARD_MODE	0x07A0
#define I2C_SET_FAST_MODE	0x07A1


struct i2c_algo_wmt_data {
       int  (*write_msg)(void *priv_data, unsigned int slave_addr, char *buf, unsigned int length , int restart, int last) ;
       int  (*read_msg)(void *priv_data, unsigned int slave_addr, char *buf, unsigned int length , int restart, int last) ;
       int  (*wait_bus_not_busy)(void *priv_data);
       void (*reset) (void *priv_data);
       void (*set_mode)(void *priv_data, enum i2c_mode_e) ;
       int  udelay;
       int  timeout;
	void *priv_data;
};

#endif
