/*
 * drivers/mtd/nand/wmt_nand.h
 *
 * Copyright (C) 2008 WonderMedia Technologies, Inc.
 * Copyright (C) 2010 Alexey Charkov <alchark@gmail.com>
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#ifndef __NFC_H__
#define __NFC_H__

#if (CONFIG_MTD_NAND_HM_ECC == 1)
 #if (CONFIG_MTD_NAND_PAGE_SIZE != 512)
#define NAND_BBT_BCH_ECC
#else
#undef NAND_BBT_BCH_ECC
#endif

#else
#undef NAND_BBT_BCH_ECC
#endif

/*
 * Status Registers
 */
#define VT8500_NFC_DATAPORT			0x00

/*
 * Control Registers
 */
#define VT8500_NFC_COMCTRL			0x04
#define VT8500_NFC_COMPORT0			0x08
#define VT8500_NFC_COMPORT1_2			0x0c
#define VT8500_NFC_COMPORT3_4			0x10
#define VT8500_NFC_COMPORT5_6			0x14
#define VT8500_NFC_COMPORT7			0x18
#define VT8500_NFC_COMPORT8_9			0x1c
#define VT8500_NFC_DMA_COUNTER			0x20
#define VT8500_NFC_SMC_ENABLE			0x24

/*
 * Status Registers
 */
#define VT8500_NFC_MISC_STAT_PORT		0x28

/*
 *  Status Control Registers
 */
#define VT8500_NFC_HOST_STAT_CHANGE		0x2c

/*
 * Control Registers
 */
#define VT8500_NFC_SMC_DMA_COUNTER		0x30
#define VT8500_NFC_CALC_CTRL			0x34
#define VT8500_NFC_CALC_NUM			0x38
#define VT8500_NFC_CALC_NUM_QU			0x3c
#define VT8500_NFC_REMAINDER			0x40
#define VT8500_NFC_CHIP_ENABLE_CTRL		0x44
#define VT8500_NFC_NAND_TYPE_SEL		0x48
#define VT8500_NFC_REDUNT_ECC_STAT_MASK		0x4c
#define VT8500_NFC_READ_CYCLE_PULE_CTRL		0x50
#define VT8500_NFC_MISC_CTRL			0x54
#define VT8500_NFC_DUMMY_CTRL			0x58
#define VT8500_NFC_PAGESIZE_DIVIDER_SEL		0x5c
#define VT8500_NFC_RW_STROBE_TUNE		0x60
#define VT8500_NFC_BANK18_ECC_STAT_MASK		0x64

/*
 * Status Registers
 */
#define VT8500_NFC_ODD_BANK_PARITY_STAT		0x68
#define VT8500_NFC_EVEN_BANK_PARITY_STAT	0x6c
#define VT8500_NFC_REDUNT_AREA_PARITY_STAT	0x70
#define VT8500_NFC_IDLE_STAT			0x74
#define VT8500_NFC_PHYS_ADDR			0x78

/*
 *  Status Control Registers
 */
#define VT8500_NFC_REDUNT_ECC_STAT		0x7c
#define VT8500_NFC_BANK18_ECC_STAT		0x80

/*
 * Control Registers
 */
#define VT8500_NFC_TIMER_COUNTER_CONFIG		0x84
#define VT8500_NFC_NANDFLASH_BOOT		0x88
#define VT8500_NFC_ECC_BCH_CTRL			0x8c
#define VT8500_NFC_ECC_BCH_INT_MASK		0x90

/*
 *  Status Control Registers
 */
#define VT8500_NFC_ECC_BCH_INT_STAT1		0x94

/*
 * Status Registers
 */
#define VT8500_NFC_ECC_BCH_INT_STAT2		0x98
#define VT8500_NFC_ECC_BCH_ERR_POS1		0x9c
#define VT8500_NFC_ECC_BCH_ERR_POS2		0xa0
#define VT8500_NFC_ECC_BCH_ERR_POS3		0xa4
#define VT8500_NFC_ECC_BCH_ERR_POS4		0xa8
#define VT8500_NFC_ECC_BCH_ERR_POS5		0xac
#define VT8500_NFC_ECC_BCH_ERR_POS6		0xb0
#define VT8500_NFC_ECC_BCH_ERR_POS7		0xb4
#define VT8500_NFC_ECC_BCH_ERR_POS8		0xb8

#define ECC_FIFO_0	0xc0
#define ECC_FIFO_1	0xc4
#define ECC_FIFO_2	0xc8
#define ECC_FIFO_3	0xcc
#define ECC_FIFO_4	0xd0
#define ECC_FIFO_5	0xd4
#define ECC_FIFO_6	0xd8
#define ECC_FIFO_7	0xdc
#define ECC_FIFO_8	0xe0
#define ECC_FIFO_9	0xe4
#define ECC_FIFO_a	0xe8
#define ECC_FIFO_b	0xec
#define ECC_FIFO_c	0xf0
#define ECC_FIFO_d	0xf4
#define ECC_FIFO_e	0xf8
#define ECC_FIFO_f	0xfc

/*
 *	NAND PDMA
 */
#define NAND_DESC_BASE_ADDR	0x00D00000

#define NFC_DMA_GCR		0x100
#define NFC_DMA_IER		0x104
#define NFC_DMA_ISR		0x108
#define NFC_DMA_DESPR		0x10C
#define NFC_DMA_RBR		0x110
#define NFC_DMA_DAR		0x114
#define NFC_DMA_BAR		0x118
#define NFC_DMA_CPR		0x11C
#define NFC_DMA_CCR		0X120

/*
 *	NAND PDMA - DMA_GCR : DMA Global Control Register
 */
#define NAND_PDMA_GCR_DMA_EN	0x00000001 /* [0] -- DMA controller enable */
#define NAND_PDMA_GCR_SOFTRESET	0x00000100 /* [8] -- Software rest */

/*
 *	NAND PDMA - DMA_IER : DMA Interrupt Enable Register
 */
#define NAND_PDMA_IER_INT_EN	0x00000001 /* [0] -- DMA interrupt enable */
/*
 *	NAND PDMA - DMA_ISR : DMA Interrupt Status Register
 */
#define NAND_PDMA_IER_INT_STS	0x00000001 /* [0] -- DMA interrupt status */
/*
 *	NAND PDMA - DMA_DESPR : DMA Descriptor base address Pointer Register
 */

/*
 *	NAND PDMA - DMA_RBR : DMA Residual Bytes Register
 */
#define NAND_PDMA_RBR_End	0x80000000 /* [31] -- DMA descriptor end flag */
#define NAND_PDMA_RBR_Format	0x40000000 /* [30] -- DMA descriptor format */
/*
 *	NAND PDMA - DMA_DAR : DMA Data Address Register
 */

/*
 *	NAND PDMA - DMA_BAR : DMA Rbanch Address Register
 */

/*
 *	NAND PDMA - DMA_CPR : DMA Command Pointer Register
 */

/*
 *	NAND PDMA - DMA_CCR : DMAContext Control Register for Channel 0
 */
#define NAND_PDMA_READ			0x00
#define NAND_PDMA_WRITE			0x01
#define NAND_PDMA_CCR_RUN		0x00000080
#define NAND_PDMA_CCR_IF_to_peripheral	0x00000000
#define NAND_PDMA_CCR_peripheral_to_IF	0x00400000
#define NAND_PDMA_CCR_EvtCode		0x0000000f
#define NAND_PDMA_CCR_Evt_no_status	0x00000000
#define NAND_PDMA_CCR_Evt_ff_underrun	0x00000001
#define NAND_PDMA_CCR_Evt_ff_overrun	0x00000002
#define NAND_PDMA_CCR_Evt_desp_read	0x00000003
#define NAND_PDMA_CCR_Evt_data_rw	0x00000004
#define NAND_PDMA_CCR_Evt_early_end	0x00000005
#define NAND_PDMA_CCR_Evt_success	0x0000000f

/*
 *	PDMA Descriptor short
 */
struct _NAND_PDMA_DESC_S{
	unsigned long volatile ReqCount : 16;	/* bit 0 -15 Request count */
	unsigned long volatile i : 1;		/* bit 16    interrupt */
	unsigned long volatile reserve : 13;	/* bit 17-29 reserved */
	unsigned long volatile format : 1;	/* bit 30    The descriptor format */
	unsigned long volatile end : 1;		/* bit 31    End flag of descriptor list*/
	unsigned long volatile DataBufferAddr : 32; /* bit 31 Data Buffer address */
};

/*
 *	PDMA Descriptor long
 */
struct _NAND_PDMA_DESC_L{
	unsigned long volatile ReqCount : 16;	/* bit 0 -15 Request count */
	unsigned long volatile i : 1;		/* bit 16    interrupt */
	unsigned long volatile reserve : 13;	/* bit 17-29 reserved */
	unsigned long volatile format : 1;	/* bit 30    The descriptor format */
	unsigned long volatile end : 1;		/* bit 31    End flag of descriptor list*/
	unsigned long volatile DataBufferAddr : 32;/* bit 31-0 Data Buffer address */
	unsigned long volatile BranchAddr : 32;	/* bit 31-2  Descriptor Branch address */
	unsigned long volatile reserve0 : 32;	/* bit 31-0  reserved */
};

/* cfg_15 */
#define USE_SW_ECC 0x04
#define USE_HW_ECC 0


#define ECC1bit		0
#define ECC4bit		1
#define ECC8bit		2
#define ECC12bit	3
#define ECC16bit	4

/* cfg_23 */
#define READ_RESUME	0x100
#define ECC_MODE	3

/* ECC BCH interrupt mask */
#define eccBCH_inetrrupt_enable		0x101
#define eccBCH_inetrrupt_disable	0x0


/* cfg_1 */
#define DPAHSE_DISABLE	0x80	/* disable data phase*/
#define NAND2NFC	0x40	/* direction : nand to controller */
#define NFC2NAND	0x00	/* direction : controller to controller */
#define SING_RW		0x20	/* enable signal read/ write command */
#define MUL_CMDS	0x10	/* support cmd+addr+cmd */
#define NFC_TRIGGER	0x01	/* start cmd&addr sequence */


/* cfg_a */
#define NFC_CMD_RDY	0x04
#define NFC_BUSY	0x02	/* command and data is being transfer in flash I/O */
#define FLASH_RDY	0x01	/* flash is ready */

/* cfg_b */
#define B2R		0x08	/* status form busy to ready */

/*cfg_12 */
#define WP_DISABLE	(1<<4)	/* disable write protection */
#define OLDDATA_EN	(1<<2)	/* enable old data structure */
#define WIDTH_8		0
#define WIDTH_16	(1<<3)
#define PAGE_512	0
#define PAGE_2K		1
#define PAGE_4K		2
#define DIRECT_MAP	(1<<5)
#define CHECK_ALLFF	(1<<6)

/* cfg_1d */
#define NFC_IDLE	0x01


/*cfg_25 status */
#define ERR_CORRECT	0x100
#define BCH_ERR		0x1

/*cfg_26 status */
#define BCH_ERR_CNT	0xF

#define ADDR_COLUMN		1
#define ADDR_PAGE		2
#define ADDR_COLUMN_PAGE	3
#define WRITE_NAND_COMMAND(d, adr) do { *(volatile unsigned char *)(adr) = (unsigned char)(d); } while (0)
#define WRITE_NAND_ADDRESS(d, adr) do { *(volatile unsigned char *)(adr) = (unsigned char)(d); } while (0)




#define DMA_SINGNAL	0
#define DMA_INC4	0x10
#define DMA_INC8	0x20

int nand_init_pdma(struct mtd_info *mtd);
int nand_free_pdma(struct mtd_info *mtd);
int nand_alloc_desc_pool(unsigned long *DescAddr);
int nand_init_short_desc(unsigned long *DescAddr, unsigned int ReqCount, unsigned long *BufferAddr);
int nand_init_long_desc(unsigned long *DescAddr, unsigned int ReqCount, unsigned long *BufferAddr,
unsigned long *BranchAddr, int End);
int nand_config_pdma(struct mtd_info *mtd, unsigned long *DescAddr, unsigned int dir);
int nand_pdma_handler(struct mtd_info *mtd);
void nand_hamming_ecc_1bit_correct(struct mtd_info *mtd);
void bch4bit_data_ecc_correct(struct mtd_info *mtd);
void bch4bit_redunt_ecc_correct(struct mtd_info *mtd);

extern uint32_t NFC_RWTimming;
extern uint32_t NFC_ClockDivisor;
extern uint32_t NFC_ClockMask;
extern uint32_t ECC8BIT_ENGINE;
extern uint32_t redunt_err_mark;
extern uint32_t chip_swap;
#endif
